@@ -6,6 +6,7 @@
 from __future__ import unicode_literals
 
 import contextlib
+import functools
 import io
 import os
 import re
@@ -422,13 +423,12 @@ import os, math, subprocess
                 unused_module=['foo.abc', 'foo.subprocess',
                                'foo.math']))
 
-    def test_filter_code_should_ignore_multiline_imports(self):
+    def test_filter_code_multiline_imports(self):
         self.assertEqual(
             r"""\
 import os
 pass
-import os, \
-    math, subprocess
+import os
 os.foo()
 """,
             ''.join(autoflake.filter_code(r"""\
@@ -437,6 +437,39 @@ import re
 import os, \
     math, subprocess
 os.foo()
+""")))
+
+    def test_filter_code_multiline_from_imports(self):
+        self.assertEqual(
+            r"""\
+import os
+pass
+from os.path import (
+    join,
+)
+join('a', 'b')
+pass
+os.foo()
+from os.path import \
+    isdir
+isdir('42')
+""",
+            ''.join(autoflake.filter_code(r"""\
+import os
+import re
+from os.path import (
+    exists,
+    join,
+)
+join('a', 'b')
+from os.path import \
+    abspath, basename, \
+    commonpath
+os.foo()
+from os.path import \
+    isfile \
+    , isdir
+isdir('42')
 """)))
 
     def test_filter_code_should_ignore_semicolons(self):
@@ -1362,7 +1395,7 @@ except ImportError:
             autoflake._main(argv=['my_fake_program', '--check', filename],
                             standard_out=output_file,
                             standard_error=None)
-            self.assertEqual('No issues detected!', output_file.getvalue())
+            self.assertEqual('No issues detected!\n', output_file.getvalue())
 
     def test_check_correct_file(self):
         with temporary_file("""\
@@ -1374,7 +1407,7 @@ print(x)
             autoflake._main(argv=['my_fake_program', '--check', filename],
                             standard_out=output_file,
                             standard_error=None)
-            self.assertEqual('No issues detected!', output_file.getvalue())
+            self.assertEqual('No issues detected!\n', output_file.getvalue())
 
     def test_check_useless_pass(self):
         with temporary_file("""\
@@ -1624,6 +1657,473 @@ print(x)
                 process.communicate()[1].decode())
 
 
+class MultilineFromImportTests(unittest.TestCase):
+    def test_is_over(self):
+        filt = autoflake.FilterMultilineImport('from . import (\n')
+        self.assertTrue(filt.is_over('module)\n'))
+        self.assertTrue(filt.is_over('  )\n'))
+        self.assertTrue(filt.is_over('  )  # comment\n'))
+        self.assertTrue(filt.is_over('from module import (a, b)\n'))
+        self.assertFalse(filt.is_over('#  )'))
+        self.assertFalse(filt.is_over('module\n'))
+        self.assertFalse(filt.is_over('module, \\\n'))
+        self.assertFalse(filt.is_over('\n'))
+
+        filt = autoflake.FilterMultilineImport('from . import module, \\\n')
+        self.assertTrue(filt.is_over('module\n'))
+        self.assertTrue(filt.is_over('\n'))
+        self.assertTrue(filt.is_over('m1, m2  # comment with \\\n'))
+        self.assertFalse(filt.is_over('m1, m2 \\\n'))
+        self.assertFalse(filt.is_over('m1, m2 \\  #\n'))
+        self.assertFalse(filt.is_over('m1, m2 \\  # comment with \\\n'))
+        self.assertFalse(filt.is_over('\\\n'))
+
+        # "Multiline" imports that are not really multiline
+        filt = autoflake.FilterMultilineImport('import os; '
+                                               'import math, subprocess')
+        self.assertTrue(filt.is_over())
+
+    unused = ()
+
+    def assert_fix(self, lines, result, remove_all=True):
+        fixer = autoflake.FilterMultilineImport(
+            lines[0],
+            remove_all_unused_imports=remove_all,
+            unused_module=self.unused
+        )
+        fixed = functools.reduce(lambda acc, x: acc(x), lines[1:], fixer())
+        self.assertEqual(fixed, result)
+
+    def test_fix(self):
+        self.unused = ['third_party.lib' + str(x) for x in (1, 3, 4)]
+
+        # Example m0 (isort)
+        self.assert_fix([
+            'from third_party import (lib1, lib2, lib3,\n',
+            '                         lib4, lib5, lib6)\n'
+        ],
+            'from third_party import (lib2, lib5, lib6)\n'
+        )
+
+        # Example m1(isort)
+        self.assert_fix([
+            'from third_party import (lib1,\n',
+            '                         lib2,\n',
+            '                         lib3,\n',
+            '                         lib4,\n',
+            '                         lib5,\n',
+            '                         lib6)\n'
+        ],
+            'from third_party import (lib2,\n'
+            '                         lib5,\n'
+            '                         lib6)\n'
+        )
+
+        # Variation m1(isort)
+        self.assert_fix([
+            'from third_party import (lib1\n',
+            '                        ,lib2\n',
+            '                        ,lib3\n',
+            '                        ,lib4\n',
+            '                        ,lib5\n',
+            '                        ,lib6)\n'
+        ],
+            'from third_party import (lib2\n'
+            '                        ,lib5\n'
+            '                        ,lib6)\n'
+        )
+
+        # Example m2 (isort)
+        self.assert_fix([
+            'from third_party import \\\n',
+            '    lib1, lib2, lib3, \\\n',
+            '    lib4, lib5, lib6\n'
+        ],
+            'from third_party import \\\n'
+            '    lib2, lib5, lib6\n'
+        )
+
+        # Example m3 (isort)
+        self.assert_fix([
+            'from third_party import (\n',
+            '    lib1,\n',
+            '    lib2,\n',
+            '    lib3,\n',
+            '    lib4,\n',
+            '    lib5\n',
+            ')\n'
+        ],
+            'from third_party import (\n'
+            '    lib2,\n'
+            '    lib5\n'
+            ')\n'
+        )
+
+        # Example m4 (isort)
+        self.assert_fix([
+            'from third_party import (\n',
+            '    lib1, lib2, lib3, lib4,\n',
+            '    lib5, lib6)\n'
+        ],
+            'from third_party import (\n'
+            '    lib2, lib5, lib6)\n'
+        )
+
+        # Example m5 (isort)
+        self.assert_fix([
+            'from third_party import (\n',
+            '    lib1, lib2, lib3, lib4,\n',
+            '    lib5, lib6\n',
+            ')\n'
+        ],
+            'from third_party import (\n'
+            '    lib2, lib5, lib6\n'
+            ')\n'
+        )
+
+        # Some Deviations
+        self.assert_fix([
+            'from third_party import (\n',
+            '    lib1\\\n',  # only unused + line continuation
+            '    ,lib2, \n',
+            '    libA\n',  # used import with no commas
+            '    ,lib3, \n',  # leading and trailing commas with unused import
+            '    libB, \n',
+            '    \\\n',  # empty line with continuation
+            '    lib4,\n',  # unused import with comment
+            ')\n'
+        ],
+            'from third_party import (\n'
+            '    lib2\\\n'
+            '    ,libA, \n'
+            '    libB,\n'
+            ')\n',
+        )
+
+        self.assert_fix([
+            'from third_party import (\n',
+            '    lib1\n',
+            ',\n',
+            '    lib2\n',
+            ',\n',
+            '    lib3\n',
+            ',\n',
+            '    lib4\n',
+            ',\n',
+            '    lib5\n',
+            ')\n'
+        ],
+            'from third_party import (\n'
+            '    lib2\n'
+            ',\n'
+            '    lib5\n'
+            ')\n'
+        )
+
+        self.assert_fix([
+            'from third_party import (\n',
+            '    lib1 \\\n',
+            ', \\\n',
+            '    lib2 \\\n',
+            ',\\\n',
+            '    lib3\n',
+            ',\n',
+            '    lib4\n',
+            ',\n',
+            '    lib5 \\\n',
+            ')\n'
+        ],
+            'from third_party import (\n'
+            '    lib2 \\\n'
+            ', \\\n'
+            '    lib5 \\\n'
+            ')\n'
+        )
+
+    def test_indentation(self):
+        # Some weird indentation examples
+        self.unused = ['third_party.lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            '    from third_party import (\n',
+            '            lib1, lib2, lib3, lib4,\n',
+            '    lib5, lib6\n',
+            ')\n'
+        ],
+            '    from third_party import (\n'
+            '            lib2, lib5, lib6\n'
+            ')\n'
+        )
+        self.assert_fix([
+            '\tfrom third_party import \\\n',
+            '\t\tlib1, lib2, lib3, \\\n',
+            '\t\tlib4, lib5, lib6\n'
+        ],
+            '\tfrom third_party import \\\n'
+            '\t\tlib2, lib5, lib6\n'
+        )
+
+    def test_fix_relative(self):
+        # Example m0 (isort)
+        self.unused = ['.lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'from . import (lib1, lib2, lib3,\n',
+            '               lib4, lib5, lib6)\n'
+        ],
+            'from . import (lib2, lib5, lib6)\n'
+        )
+
+        # Example m1(isort)
+        self.unused = ['..lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'from .. import (lib1,\n',
+            '                lib2,\n',
+            '                lib3,\n',
+            '                lib4,\n',
+            '                lib5,\n',
+            '                lib6)\n'
+        ],
+            'from .. import (lib2,\n'
+            '                lib5,\n'
+            '                lib6)\n'
+        )
+
+        # Example m2 (isort)
+        self.unused = ['...lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'from ... import \\\n',
+            '    lib1, lib2, lib3, \\\n',
+            '    lib4, lib5, lib6\n'
+        ],
+            'from ... import \\\n'
+            '    lib2, lib5, lib6\n'
+        )
+
+        # Example m3 (isort)
+        self.unused = ['.parent.lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'from .parent import (\n',
+            '    lib1,\n',
+            '    lib2,\n',
+            '    lib3,\n',
+            '    lib4,\n',
+            '    lib5\n',
+            ')\n'
+        ],
+            'from .parent import (\n'
+            '    lib2,\n'
+            '    lib5\n'
+            ')\n'
+        )
+
+    def test_fix_without_from(self):
+        self.unused = ['lib' + str(x) for x in (1, 3, 4)]
+
+        # Multiline but not "from"
+        self.assert_fix([
+            'import \\\n',
+            '    lib1, lib2, lib3 \\\n',
+            '    ,lib4, lib5, lib6\n'
+        ],
+            'import \\\n'
+            '    lib2, lib5, lib6\n'
+        )
+        self.assert_fix([
+            'import lib1, lib2, lib3, \\\n',
+            '       lib4, lib5, lib6\n'
+        ],
+            'import lib2, lib5, lib6\n'
+        )
+
+        # Problematic example without "from"
+        self.assert_fix([
+            'import \\\n',
+            '    lib1,\\\n',
+            '    lib2, \\\n',
+            '    libA\\\n',  # used import with no commas
+            '    ,lib3, \\\n',  # leading and trailing commas with unused
+            '    libB, \\\n',
+            '    \\  \n',  # empty line with continuation
+            '    lib4\\\n',  # unused import with comment
+            '\n'
+        ],
+            'import \\\n'
+            '    lib2,\\\n'
+            '    libA, \\\n'
+            '    libB\\\n'
+            '\n'
+        )
+
+        self.unused = ['lib{}.x.y.z'.format(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'import \\\n',
+            '    lib1.x.y.z \\',
+            '    , \\\n',
+            '    lib2.x.y.z \\\n',
+            '    , \\\n',
+            '    lib3.x.y.z \\\n',
+            '    , \\\n',
+            '    lib4.x.y.z \\\n',
+            '    , \\\n',
+            '    lib5.x.y.z\n'
+        ],
+            'import \\\n'
+            '    lib2.x.y.z \\'
+            '    , \\\n'
+            '    lib5.x.y.z\n'
+        )
+
+    def test_give_up(self):
+        # Semicolon
+        self.unused = ['lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'import \\\n',
+            '    lib1, lib2, lib3, \\\n',
+            '    lib4, lib5; import lib6\n'
+        ],
+            'import \\\n'
+            '    lib1, lib2, lib3, \\\n'
+            '    lib4, lib5; import lib6\n'
+        )
+        # Comments
+        self.unused = ['.lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'from . import ( # comment\n',
+            '    lib1,\\\n',  # only unused + line continuation
+            '    lib2, \n',
+            '    libA\n',  # used import with no commas
+            '    ,lib3, \n',  # leading and trailing commas with unused import
+            '    libB, \n',
+            '    \\  \n',  # empty line with continuation
+            '    lib4,  # noqa \n',  # unused import with comment
+            ') ; import sys\n'
+        ],
+            'from . import ( # comment\n'
+            '    lib1,\\\n'
+            '    lib2, \n'
+            '    libA\n'
+            '    ,lib3, \n'
+            '    libB, \n'
+            '    \\  \n'
+            '    lib4,  # noqa \n'
+            ') ; import sys\n'
+        )
+
+    def test_just_one_import_used(self):
+        self.unused = ['lib2']
+        self.assert_fix([
+            'import \\\n',
+            '    lib1\n'
+        ],
+            'import \\\n'
+            '    lib1\n'
+        )
+        self.assert_fix([
+            'import \\\n',
+            '    lib2\n'
+        ],
+            'pass\n'
+        )
+        # Example from issue #8
+        self.unused = ['re.subn']
+        self.assert_fix([
+            '\tfrom re import (subn)\n',
+        ],
+            '\tpass\n',
+        )
+
+    def test_just_one_import_left(self):
+        # Examples from issue #8
+        self.unused = ['math.sqrt']
+        self.assert_fix([
+            'from math import (\n',
+            '        sqrt,\n',
+            '        log\n',
+            '    )\n'
+        ],
+            'from math import (\n'
+            '        log\n'
+            '    )\n'
+        )
+        self.unused = ['module.b']
+        self.assert_fix([
+            'from module import (a, b)\n',
+        ],
+            'from module import a\n',
+        )
+        self.assert_fix([
+            'from module import (a,\n',
+            '                    b)\n',
+        ],
+            'from module import a\n',
+        )
+        self.unused = []
+        self.assert_fix([
+            'from re import (subn)\n',
+        ],
+            'from re import (subn)\n',
+        )
+
+    def test_no_empty_imports(self):
+        self.unused = ['lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'import \\\n',
+            '    lib1, lib3, \\\n',
+            '    lib4 \n'
+        ],
+            'pass \n'
+        )
+
+        # Indented parenthesized block
+        self.unused = ['.parent.lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            '\t\tfrom .parent import (\n',
+            '    lib1,\n',
+            '    lib3,\n',
+            '    lib4,\n',
+            ')\n'
+        ],
+            '\t\tpass\n'
+        )
+
+    def test_without_remove_all(self):
+        self.unused = ['lib' + str(x) for x in (1, 3, 4)]
+        self.assert_fix([
+            'import \\\n',
+            '    lib1,\\\n',
+            '    lib3,\\\n',
+            '    lib4\n',
+        ],
+            'import \\\n'
+            '    lib1,\\\n'
+            '    lib3,\\\n'
+            '    lib4\n',
+            remove_all=False
+        )
+
+        self.unused += ['os.path.' + x for x in ('dirname', 'isdir', 'join')]
+        self.assert_fix([
+            'from os.path import (\n',
+            '    dirname,\n',
+            '    isdir,\n',
+            '    join,\n',
+            ')\n'
+        ],
+            'pass\n',
+            remove_all=False
+        )
+        self.assert_fix([
+            'import \\\n',
+            '    os.path.dirname, \\\n',
+            '    lib1, \\\n',
+            '    lib3\n',
+        ],
+            'import \\\n'
+            '    lib1, \\\n'
+            '    lib3\n',
+            remove_all=False
+        )
+
+
 @contextlib.contextmanager
 def temporary_file(contents, directory='.', suffix='.py', prefix=''):
     """Write contents to temporary file and yield it."""
